
/* table_scripter */
/* Made available through the courtesy of NASA's Goddard Space Flight Center */
/*-----------------------------------------------------------------------------

     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2
     of the License, or (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program (at the end); if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  PROGRAM: 	TABLE_SCRIPTER			COMPLETED: 01/19/93

  PROGRAMMER: 	Phillip C. Brisco

  VERSION:	1.8  This version has been ported from Fortran to C.

  FUNCTION:	This subroutine saves data from a table to an SQL script file.

  It uses the foreign command to get the command line entry to determine the
  username, database name, and tablename to process.  The following switches
  can be entered from the command line:
    -b  fill each line with data (best fit)
    -c  chosen fields
    -d  database name
    -f  create a command file
    -h  show a list of command line switches
    -l  simple list output.  The file has an extension of '.lis'.  Subparameter
        j left justifies the text and subparameter q uses quotes vice
	apostrophes for delimiting strings
    -o  suppresses command file message
    -r  user entered remarks for output file
    -s  script file
    -t  tablename
    -u  username
    -v  suppresses non-error messages
    -w  where clause (must be delimited by ")

  To use this program, a symbol has to be set up to run it (e.g.
  ugga :== $usr1:[brisco.tmp]table_scripter).

  The symbol is executed with the switches (e.g. ugga -ubrisco -dbrisco_pdb
  -tcurrent_build).

  The -d and -t switches are the only required switches.  If the '-u' switch
  is omitted, the program defaults to the current user's name.  Switches can be
  entered in any order, but there must be no space between the switch and the
  following name.  The -w, -c, and -r  switches are strings which must be
  delimited by the double quote marks (e.g. -w"filename = 'UGGA'"
  -c"This is a test").

  MODIFICATIONS:

  1.1   Phillip C. Brisco	02/03/93	Added more error checking to
						user input and added the -v
						switch.

  1.2	Phillip C. Brisco	03/19/93	Added the -o switch.

  1.3	Phillip C. Brisco	05/12/93	Added the RECL = 512 option to
						the open statement so that
						fields > 128 in length can be
						written out to a file.  This
						should be made more dynamic in
						the future.

  1.4	Phillip C. Brisco	05/19/93	Added test for the single quote
						so that user entered single
						quotes will be copied to the
						database.

  1.5	Phillip C. Brisco	05/20/93	Changed the conversion from
						integer and real to strings so
						that the code is non-VAX
						specific.  Also made the
						record length of the output
						file dynamic.

  1.6	Phillip C. Brisco	05/24/93	Put all system specific code
						into subroutines starting with
						the letter x.  Added the help,
						comment and create command file
						switches.  Added date/time
						stamp info to output sql file.

  1.7	Phillip C. Brisco	07/15/93	Rearranged code so that an
						output file is not created if
						no data is retrieved.

  1.8	Phillip C. Brisco	11/17/93	Added ability for user to
						select fields to be output.
						Also added the option to list
						the table in simple ASCII
						format.
*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <jpidef.h>;
#include <ssdef.h>;
#include <libdef.h>;
#include <descrip.h>;

/* In order to use the SQLCA, we have to declare a variable (which we have
   named sqlca) of type IISQLCA, since the SQLCA structure has been TYPEDEFed
   with the name of IISQLCA.  The same holds true for the SQLDA structure.
   */
EXEC SQL INCLUDE SQLCA;
IISQLCA sqlca;
EXEC SQL INCLUDE SQLDA;
IISQLDA results;
struct sqda_ *res_ptr = &results;

#define max_i_o_length 200
#define max_out 300
#define true 1;
#define false 0;

EXEC SQL BEGIN DECLARE SECTION;
  long int i, j, k, l, k1;
  long int where_begin, where_end, table_begin, table_end;
  long int num_digits, number_of_columns, record_length;
  long int db_length, command_line_length, column_length;
  long int fs_start, fs_end;
  long int column_output_length;
  long int total_number_of_rows;
  char single_quote[1], version_num[5] = "1.80", the_time[6];
  char the_date[10], dbname[16], column_name[33], select_table_name[33];
  char current_user[33], file_name[33], com_file_name[33], the_user[33];
  char err_msg[80], column_output[80], buffer[200], fields_selected[200];
  char where_clause[200], script_clause[200], command_line[200];
  char out_put[400], data_output[400], tmp_put[400];

  struct field_types {
    int null_ind[max_i_o_length];
    long int integer_number[max_i_o_length];
    double real_number[max_i_o_length];
    char character_data[200][max_i_o_length];
  } b[10];
EXEC SQL END DECLARE SECTION;

long int list_begin, list_end, comment_begin, comment_end;
long int script_begin, script_end;
int the_logical_length;
long int istat, normal_ret;
short int write_the_junque;
short int uflag, tflag, dflag, wflag, exit_flag, verbose_off, sflag;
short int command_line_off, create_command_file, best_fit;
short int user_added_text, help_me, list_format, left_justify;
short int double_quote;
char **argv_ptr;

FILE *iicolumns_chan;

void xget_jpi (char *cuser,long *cistat,long *cnormal_ret);
void dynamic_open(void);

/****************************************************************************/
/* Start of executable code */

main(int argc, char *argv[])
{
  exit_flag = false;
  uflag = false;
  tflag = false;
  dflag = false;
  wflag = false;
  sflag = false;
  verbose_off = false;
  command_line_off = false;
  create_command_file = false;
  user_added_text = false;
  best_fit = false;
  list_format = false;
  left_justify = false;
  double_quote = false;

/*--------------------- Beginning of test for command line input -----------*/
  sprintf (fields_selected,"*");
  fs_start = 1;
  fs_end = 1;

  for (i = 1;i < argc;i++) {
    argv_ptr = &argv[i];

    /*  Database owner. */
    if ((**argv_ptr == '-' && (*(*argv_ptr + 1) == 'u' ||
			       *(*argv_ptr + 1) == 'U')) && !uflag) {

      /* Lowercase the argument
       */
      for (j = 0;j < strlen(*argv_ptr);j++) {
	if (*(*argv_ptr + j) >= 'A' && *(*argv_ptr + j) <= 'Z') {
	  **(argv_ptr + j) = *(*argv_ptr + j) | 32;
	}
      }

      if (strlen(*argv_ptr) > 2) {
	strcpy(current_user,*argv_ptr + 2);
	uflag = true;
      }
    } else if ((**argv_ptr == '-' &&
		  (*(*argv_ptr + 1) == 'u' || *(*argv_ptr + 1) == 'U'))
	       && uflag) {
	printf ("Multiple users specified in command line.\n");
	exit_flag = true;
      }

    /*  Database table. */
    if ((**argv_ptr == '-' && (*(*argv_ptr + 1) == 't' ||
			       *(*argv_ptr + 1) == 'T')) && !tflag) {

      /* Lowercase the argument
       */
      for (j = 0;j < strlen(*argv_ptr);j++) {
	if (*(*argv_ptr + j) >= 'A' && *(*argv_ptr + j) <= 'Z') {
	  **(argv_ptr + j) = *(*argv_ptr + j) | 32;
	}
      }

      if (strlen(*argv_ptr) > 2) {
	strcpy(select_table_name,*argv_ptr + 2);
	tflag = true;
      }
    } else if ((**argv_ptr == '-' &&
		  (*(*argv_ptr + 1) == 't' || *(*argv_ptr + 1) == 'T'))
	       && tflag) {
	printf ("Multiple tables specified in command line.\n");
	exit_flag = true;
      }

    /* Database name.
       */
    if ((**argv_ptr == '-' && (*(*argv_ptr + 1) == 'd' ||
			       *(*argv_ptr + 1) == 'D')) && !dflag) {

      /* Lowercase the argument
       */
      for (j = 0;j < strlen(*argv_ptr);j++) {
	if (*(*argv_ptr + j) >= 'A' && *(*argv_ptr + j) <= 'Z') {
	  **(argv_ptr + j) = *(*argv_ptr + j) | 32;
	}
      }

      if (strlen(*argv_ptr) > 2) {
	strcpy(dbname,*argv_ptr + 2);
	dflag = true;
      }
    } else if ((**argv_ptr == '-' &&
		(*(*argv_ptr + 1) == 'd' || *(*argv_ptr + 1) == 'D'))
	       && dflag) {
	printf ("Multiple databases specified on command line.\n");
	exit_flag = true;
      }

    /* Where clause.
       */
    if ((**argv_ptr == '-' && (*(*argv_ptr + 1) == 'w' ||
			       *(*argv_ptr + 1) == 'W')) && !wflag) {

      /* Lowercase the argument
       */
      for (j = 0;j < strlen(*argv_ptr);j++) {
	if (*(*argv_ptr + j) >= 'A' && *(*argv_ptr + j) <= 'Z') {
	  **(argv_ptr + j) = *(*argv_ptr + j) | 32;
	}
      }

      strcpy(where_clause," WHERE  ");
      strcpy(&where_clause + 7,*argv_ptr + 2);
      dflag = true;

    } else if ((**argv_ptr == '-' &&
		(*(*argv_ptr + 1) == 'w' || *(*argv_ptr + 1) == 'W'))
	       && wflag) {
	printf ("Multiple where clauses specified on one line.\n");
	exit_flag = true;
      }

    /* Script file.
       */
    if ((**argv_ptr == '-' && (*(*argv_ptr + 1) == 's' ||
			       *(*argv_ptr + 1) == 'S')) && !sflag) {

      /* Lowercase the argument
       */
      for (j = 0;j < strlen(*argv_ptr);j++) {
	if (*(*argv_ptr + j) >= 'A' && *(*argv_ptr + j) <= 'Z') {
	  **(argv_ptr + j) = *(*argv_ptr + j) | 32;
	}
      }

      strcpy(script_clause,*argv_ptr + 2);
      sflag = true;

    } else if ((**argv_ptr == '-' &&
		(*(*argv_ptr + 1) == 's' || *(*argv_ptr + 1) == 'S'))
	       && sflag) {
	printf ("Multiple script files specified on one line.\n");
	exit_flag = true;
      }

    /* Verbose switch
       */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'v' ||
			      *(*argv_ptr + 1) == 'V')) {
      verbose_off = true;
    }

    /* Simple list switch
     */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'l' ||
			      *(*argv_ptr + 1) == 'L')) {
      list_format = true;
      j = 2;

      while (*(*argv_ptr + j) != '"' && j < strlen(*argv_ptr)) {
	if (*(*argv_ptr + j) == 'j'  || *(*argv_ptr + j) == 'J') {
	  left_justify = true;
	}
	if (*(*argv_ptr + j) == 'q'  || *(*argv_ptr + j) == 'Q') {
	  double_quote = true;
	}
	++j;
      }

      list_begin = j;
    }

    /* Command file output switch.
       */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'o' ||
				 *(*argv_ptr + 1) == 'O')) {
      command_line_off = true;
    }

    /* Create command file switch.
       */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'f' ||
				 *(*argv_ptr + 1) == 'F')) {
      create_command_file = true;
    }

    /* Add user-entered comment switch.
       */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'r' ||
				 *(*argv_ptr + 1) == 'R')) {
      user_added_text = true;
      comment_begin = i;
    }

    /* Fields selected.
     */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'c' ||
				 *(*argv_ptr + 1) == 'C')) {
      strcpy (fields_selected,*argv_ptr + 2);
    }

    /* Best fit switch.
     */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'b' ||
				 *(*argv_ptr + 1) == 'B')) {
      best_fit = true;
    }

    /* Help switch.
     */
    if (**argv_ptr == '-' && (*(*argv_ptr + 1) == 'h' ||
			      *(*argv_ptr + 1) == 'H')) {
      help_me = true;
    }
  }
/*--------------------- End of testing for command line input ---------------*/

  if (help_me) {
    printf ("\n\nThe following are legal command line switches:\n\n");
    printf ("-b     fill each line with data (best fit)\n");
    printf ("-c     user-selected data fields\n");
    printf ("-d     database name\n");
    printf ("-f     create a command file\n");
    printf ("-h     show a list of command line switches\n");
    printf ("-l     create a simple list file.  Has an extension of '.lis'\n");
    printf ("       Subparameter j left justifies the text and subparameter");
    printf (" q uses quotes\n");
    printf ("       vice apostrophes for delimiting strings.\n");
    printf ("-o     suppresses command file message\n");
    printf ("-r     user entered comment for output file\n");
    printf ("-s     name of script file\n");
    printf ("-t     tablename\n");
    printf ("-u     username\n");
    printf ("-v     verbose mode\n");
    printf ("-w     where clause\n\n");
    return;
    }

  single_quote[0] = 39;
  xget_jpi(&the_user,&istat,&normal_ret);

  /* Lowercase the username since searches are case sensitive in INGRES.
     */
  for (i = 0;i < strlen(the_user);i++) {
    if (the_user[i] >= 'A' && the_user[i] <= 'Z') {
      the_user[i] = the_user[i] | 32;
    }
  }

  /* IF the user did not select the user flag when entering the command
     line, use the account name as the user name.
     */
  if (current_user[1] <= 32) {
    strcpy (current_user,the_user);
    uflag = true;
  }

  /* IF the user name, table name, or database name is not found, exit the
     program, else get the lengths of the aforementioned names.
     */
  if (!uflag || !tflag || !dflag || exit_flag) {
    if (!uflag) { printf ("User information not found.\n"); }
    if (!tflag) { printf ("Table information not found.\n"); }
    if (!dflag) { printf ("Database information not entered.\n"); }
    return;
  } else {
    db_length = strlen(dbname);
    the_logical_length = strlen(current_user);
    j = strlen(select_table_name);
  }

  if (!verbose_off) { printf ("Starting table_scripter...\n"); }

  EXEC SQL CONNECT :dbname IDENTIFIED BY :current_user;

  if (sqlca.sqlcode) {
    EXEC SQL INQUIRE_INGRES (:err_msg = ERRORTEXT);
    printf ("%s\n",err_msg);
    return;
  }

  if (!verbose_off) {
    printf ("\nCreating data and command files for table %s\n",
	    select_table_name);
    printf ("on the %s database owned by %s\n",dbname,current_user);
  }

  i = 0;

  if (list_format) {
    best_fit = false;
    if (sflag) {
      strcpy (file_name,script_clause);
    } else {
      EXEC SQL SELECT concat(squeeze(:select_table_name),'.lis')
	INTO :file_name;
    }
  } else {
    if (sflag) {
      strcpy (file_name,script_clause);
    } else {
      EXEC SQL SELECT concat(squeeze(:select_table_name),'.sql')
	INTO :file_name;
    }
  }

  j = 0;
  k = 0;

  for (i = 0;i < 32;i++) {
    if (file_name[i] > 32) {j = i;}
    if (file_name[i] == 46) {k = i;}
  }

  if (k == 0) {strncat (file_name,".dat",4);}
  printf ("The output file name is %s\n",file_name);
  record_length = 0;
  sprintf (buffer,"SELECT %s FROM %s %s ",fields_selected,select_table_name,
	   where_clause);

  dynamic_open();

  EXEC SQL DECLARE c1 CURSOR FOR s1;
  EXEC SQL OPEN c1;
  EXEC SQL FETCH c1 USING DESCRIPTOR :res_ptr;

  if (sqlca.sqlcode != 0)
    {
      EXEC SQL CLOSE c1;
      sqlca.sqlcode = -1;
      printf ("Output file not created due to the lack of data.\n");
    }

  EXEC SQL CLOSE c1;
  EXEC SQL OPEN c1;

  /* Create the SQL data file.
   */
  iicolumns_chan = fopen(file_name,"w");

  write_the_junque = true;
  EXEC SQL SELECT squeeze(_date(int4(dbmsinfo('_bintim')))),
    squeeze(_time(int4(dbmsinfo('_bintim'))))
    INTO :the_date, :the_time;

  fprintf (iicolumns_chan, "%s %s %s\n%s\n", "/* Table_scripter",version_num,
	   "was used to copy data from:","/*");
  fprintf (iicolumns_chan, "/*\t%s\t\t%s\n","Table:",select_table_name);
  fprintf (iicolumns_chan, "/*\t%s\t%s\n","Database:",dbname);
  fprintf (iicolumns_chan, "/*\t%s\t\t%s\n","Owner:",current_user);
  fprintf (iicolumns_chan, "/*\t%s\t\t%s\n","User:",the_user);

  if (best_fit)
    {
      fprintf (iicolumns_chan, "/*\tFormat:\t\tbest fit\n");
    } else {
      fprintf (iicolumns_chan, "/*\tFormat:\t\tnormal\n");
    }

  if (list_format)
    {
      fprintf (iicolumns_chan, "/*\tStyle:\t\tsimple list file\n");
    } else {
      fprintf (iicolumns_chan, "/*\tStyle:\t\tSQL script file\n");
    }

  fprintf (iicolumns_chan, "/*\n/* Created on %s at %s\n/*\n",the_date,
	   the_time);

  if (user_added_text)
    {
      fprintf (iicolumns_chan, "/* %s\n",argv[comment_begin] + 2);
    }

  fprintf (iicolumns_chan,"/**/\n");
  total_number_of_rows = 0;

  while (write_the_junque)
    {
      i = 0;

      EXEC SQL FETCH c1 USING DESCRIPTOR :res_ptr;

      /* SQLCODE is equal to zero only if FETCH was completed successfully.
       */
      if (sqlca.sqlcode == 0)
	{
	  write_the_junque = true;
	  total_number_of_rows++;
	} else {
	  write_the_junque = false;
	}

      if (write_the_junque)
	{
	  results.sqld = 0;
	  /* We have to test for blank spaces.  If a character variable
	     only contains blanks spaces, we assume that it is null.
	     */
	  for (j = 0;j < results.sqln;j++)
	    {
	      if (results.sqlvar[j].sqltype == -20)
		{
		  b[0].null_ind[j] = -1;
		  
		  for (k = 0;k < max_i_o_length;k++)
		    {
		      if (b[0].character_data[j][k] > 32)
			{
			  b[0].null_ind[j] = 0;
			}
		    }
		}
	      /* Adjust the sqld field of the results structure so
		 that any fields with no data at the end are not even
		 processed.
		 */
	      if (b[0].null_ind[j] == 0 && results.sqlvar
		  [j].sqlname.sqlnamec[0] != '\0') {results.sqld = j;}
	    }

	  results.sqld++;
	  column_output[0] = '\0';

	  if (!list_format) {
	    fprintf (iicolumns_chan,"INSERT INTO %s (\n",select_table_name);
	    k = 0;
	    /* Write the names of the columns to the SQL file.
	     */

	    for (j = 0;j < results.sqld;j++)
	      {
		if (b[0].null_ind[j] == 0)
		  {
		    strcpy (column_name,
			    results.sqlvar[j].sqlname.sqlnamec);

		    if (j == 0 && j == results.sqld - 1)
		      {
			strncat (column_name,")",1);
		      } else if (j != results.sqld - 1) {
			strncat (column_name,",",1);
		      } else {
			strncat (column_name,")",1);
		      }

		    if (best_fit)
		      {
			if (strlen(column_output) + strlen(column_name) < 70)
			  {
			    strncat (column_output,column_name,
				     strlen(column_name));
			  } else {
			    fprintf (iicolumns_chan,"        %s\n",
				    column_output);
			    strcpy (column_output,column_name);
			  }
		      } else {
			fprintf (iicolumns_chan, "        %s\n",column_name);
		      }
		  }

	      }
	    if (best_fit)
	      {
		fprintf (iicolumns_chan, "        %s\n", column_output);
	      }
	    column_output_length = 0;
	    fprintf (iicolumns_chan,"  VALUES ( \n");
	  }

	  data_output[0] = '\0';
	  /* Write the column data to the SQL file.
	   */
	  for (j = 0;j < results.sqld;j++)
	    {
	      if (b[0].null_ind[j] == 0 || results.sqlvar[j].sqltype > 0)
		{
		  if (abs(results.sqlvar[j].sqltype) == 30 ||
		      abs(results.sqlvar[j].sqltype) == 31 ||
		      abs(results.sqlvar[j].sqltype) == 5)
		    {
		      if (abs(results.sqlvar[j].sqltype) == 30)
			{
			  sprintf (out_put, "%\d", b[0].integer_number[j]);
			} else {
			  sprintf (out_put, "%lf", b[0].real_number[j]);
			}
		      
		    } else {
		      if (abs(results.sqlvar[j].sqltype == 20) ||
			  abs(results.sqlvar[j].sqltype == 5) ||
			  abs(results.sqlvar[j].sqltype == 3))
			{
			  if (double_quote)
			    {
			      sprintf (out_put, "%c%s%c",34,
				       b[0].character_data[j],34);
			    } else {
			      sprintf (out_put, "'%s'",
				       b[0].character_data[j]);
			    }
			}
		    }
		  if (j == 0 && j == results.sqld - 1 && !list_format)
		    {
		      strncat (out_put,")",1);
		    } else if (j != results.sqld - 1) {
		      strncat (out_put,",",1);
		    } else {
		      strncat (out_put,")",1);
		    }
		  if (best_fit)
		    {
		      if (strlen(data_output) + strlen(out_put) < 70)
			{
			  strncat (data_output,out_put,
				   strlen(out_put));
			} else {
			  fprintf (iicolumns_chan,"        %s\n",
				   data_output);
			  strcpy (data_output,out_put);
			}
		    } else if (!list_format) {
		      fprintf (iicolumns_chan, "        %s\n",out_put);
		    } else {
		      if (left_justify)
			{
			  fprintf (iicolumns_chan, "%s\n",out_put);
			} else {
			  fprintf (iicolumns_chan, "        %s\n",out_put);
			}
		    }
		}
	    }
	  if (best_fit)
	    {
	      fprintf (iicolumns_chan, "        %s\n", data_output);
	    }

	  if (list_format)
	    {
	      fprintf (iicolumns_chan,"\n");
	    }

	}
    }
  if (!list_format)
    {
      fprintf (iicolumns_chan,"/* Total Number of rows = %d\n*/\n",
	       total_number_of_rows);
    }
  fclose (iicolumns_chan);

  if (create_command_file)
    {
      EXEC SQL SELECT concat(squeeze(:select_table_name), '.com')
	INTO :com_file_name;
      iicolumns_chan = fopen(com_file_name,"w");
      if (!command_line_off)
	{
	  fprintf (iicolumns_chan,
		   "$ WRITE SYS$OUTPUT %cReloading table %s%c\n",34,
		   select_table_name,34);
	}
      fprintf (iicolumns_chan,"$SQL <%s >%s -u%s\n",file_name,
	       dbname,current_user);
      fclose (iicolumns_chan);
    }
}

/*
  Get the account name.
  */
void xget_jpi (char *the_user,long *istat,long *normal_ret)
{
  /*  Create a VMS character descriptor of the C variable 'the_account'
      and call the descriptor 'the_account_desc'.  Use the descriptor
      int the LIB$GETJPI function to get the username.  Send the name
      (which is also in the equivalence variable 'the_account') back
      to the calling function.
      */

  int i;
  char the_account[80];
  $DESCRIPTOR(the_account_desc,the_account);

  normal_ret = SS$_NORMAL;
  istat = LIB$GETJPI(&JPI$_USERNAME,NULL,NULL,NULL,&the_account_desc);

  i = 0;
  while (the_account[i] > 32 && i < 80) {i++;}
  the_account[i] = '\0';
      
  strcpy (the_user,the_account);
  return;
}

/* Sets up dynamism so that we can get data.
 */
void dynamic_open ()
{
  EXEC SQL BEGIN DECLARE SECTION;
    long int i, j, k;
    char err_msg[80];
  EXEC SQL END DECLARE SECTION;
  EXEC SQL INCLUDE SQLDA;

  /* Get the number of columns from the iicolumns table.
   */
  EXEC SQL SELECT COUNT (*)
    INTO	:i
    FROM	iicolumns
    WHERE	table_owner = :current_user
    AND	table_name = :select_table_name;

  number_of_columns = i;
  res_ptr->sqln = number_of_columns;

  /* Whenever not found then continue (sqlcod = 100 if not found) 
   */
  EXEC SQL WHENEVER NOT FOUND CONTINUE;

  /* Equate the variable buffer to 's1' using the PREPARE statement, set
     up 's1' to use the SQLDA structure through the 'results'
     record, and do a dynamic SELECT using cursors.

     If the data type is string but not CHAR, then convert it to CHAR
     since dynamic SQL has a difficult time fitting VARCHAR onto a CHAR
     host variable.

     30/-30 = integer, 31,-31 = real, 20/-20 = char, 21/-21 = varchar,
     3/-3 = date, 5/-5 = money.
     */
  EXEC SQL PREPARE s1 FROM :buffer;

  if (sqlca.sqlcode != 0)
    {
      EXEC SQL INQUIRE_INGRES (:err_msg = errortext);
      printf ("%s\n",err_msg);
    } else {

      EXEC SQL DESCRIBE s1 INTO :res_ptr;

      if (sqlca.sqlcode != 0)
	{
	  EXEC SQL INQUIRE_INGRES (:err_msg = errortext);
	    printf ("%s\n",err_msg);
	}
    }
	
  for (i = 0;i < res_ptr->sqln;i++)
    {
      if (res_ptr->sqlvar[i].sqltype == 30)
	{
	  res_ptr->sqlvar[i].sqldata = &b[0].integer_number[i];
	  record_length + = 10;
	} else if (res_ptr->sqlvar[i].sqltype == -30) {
	  res_ptr->sqlvar[i].sqldata = &b[0].integer_number[i];
	  res_ptr->sqlvar[i].sqlind = &b[0].null_ind[i];
	  record_length + = 10;
	} else if (res_ptr->sqlvar[i].sqltype == 31) {
	  res_ptr->sqlvar[i].sqldata = &b[0].real_number[i];
	  record_length + = 20;
	} else if (res_ptr->sqlvar[i].sqltype == -31) {
	  res_ptr->sqlvar[i].sqldata = &b[0].real_number[i];
	  res_ptr->sqlvar[i].sqlind = &b[0].null_ind[i];
	  record_length + = 20;
	} else if (res_ptr->sqlvar[i].sqltype == 20) {
	  res_ptr->sqlvar[i].sqldata = &b[0].character_data[i];
	  record_length + = res_ptr->sqlvar[i].sqllen;
	  res_ptr->sqlvar[i].sqllen = max_i_o_length;
	} else if (res_ptr->sqlvar[i].sqltype == -20) {
	  res_ptr->sqlvar[i].sqldata = &b[0].character_data[i];
	  record_length + = res_ptr->sqlvar[i].sqllen;
	  res_ptr->sqlvar[i].sqllen = max_i_o_length;
	  res_ptr->sqlvar[i].sqlind = &b[0].null_ind[i];
	} else if (res_ptr->sqlvar[i].sqltype == 21) {
	  res_ptr->sqlvar[i].sqldata = &b[0].character_data[i];
	  record_length + = res_ptr->sqlvar[i].sqllen;
	  res_ptr->sqlvar[i].sqllen = max_i_o_length;
	  res_ptr->sqlvar[i].sqltype = 20;
	} else if (res_ptr->sqlvar[i].sqltype == -21) {
	  res_ptr->sqlvar[i].sqldata = &b[0].character_data[i];
	  record_length + = res_ptr->sqlvar[i].sqllen;
	  res_ptr->sqlvar[i].sqllen = max_i_o_length;
	  res_ptr->sqlvar[i].sqltype = -20;
	  res_ptr->sqlvar[i].sqlind = &b[0].null_ind[i];
	} else if (res_ptr->sqlvar[i].sqltype == 3) {
	  res_ptr->sqlvar[i].sqldata = &b[0].character_data[i];
	  record_length + = res_ptr->sqlvar[i].sqllen;
	  res_ptr->sqlvar[i].sqllen = max_i_o_length;
	  res_ptr->sqlvar[i].sqltype = 20;
	} else if (res_ptr->sqlvar[i].sqltype == -3) {
	  res_ptr->sqlvar[i].sqldata = &b[0].character_data[i];
	  record_length + = res_ptr->sqlvar[i].sqllen;
	  res_ptr->sqlvar[i].sqllen = max_i_o_length;
	  res_ptr->sqlvar[i].sqltype = -20;
	  res_ptr->sqlvar[i].sqlind = &b[0].null_ind[i];
	} else if (res_ptr->sqlvar[i].sqltype == 5) {
	  res_ptr->sqlvar[i].sqldata = &b[0].real_number[i];
	  record_length + = 20;
	} else if (res_ptr->sqlvar[i].sqltype == -5) {
	  res_ptr->sqlvar[i].sqldata = &b[0].real_number[i];
	  record_length + = 20;
	  res_ptr->sqlvar[i].sqlind = &b[0].null_ind[i];
	}
      res_ptr->sqlvar[i].sqlname.sqlnamec
	[res_ptr->sqlvar[i].sqlname.sqlnamel] = '\0';
    }
  if (record_length < 128) {record_length = 128;}
}

/*

  The GNU General Public License is as follows:

                         Version 2, June 1991

     Copyright (C) 1989, 1991 Free Software Foundation, Inc.
     675 Mass Ave, Cambridge, MA 02139, USA
     
     Everyone is permitted to copy and distribute verbatim copies
     of this license document, but changing it is not allowed.

Preamble
========

   The licenses for most software are designed to take away your
freedom to share and change it.  By contrast, the GNU General Public
License is intended to guarantee your freedom to share and change free
software--to make sure the software is free for all its users.  This
General Public License applies to most of the Free Software
Foundation's software and to any other program whose authors commit to
using it.  (Some other Free Software Foundation software is covered by
the GNU Library General Public License instead.)  You can apply it to
your programs, too.

   When we speak of free software, we are referring to freedom, not
price.  Our General Public Licenses are designed to make sure that you
have the freedom to distribute copies of free software (and charge for
this service if you wish), that you receive source code or can get it
if you want it, that you can change the software or use pieces of it in
new free programs; and that you know you can do these things.

   To protect your rights, we need to make restrictions that forbid
anyone to deny you these rights or to ask you to surrender the rights.
These restrictions translate to certain responsibilities for you if you
distribute copies of the software, or if you modify it.

   For example, if you distribute copies of such a program, whether
gratis or for a fee, you must give the recipients all the rights that
you have.  You must make sure that they, too, receive or can get the
source code.  And you must show them these terms so they know their
rights.

   We protect your rights with two steps: (1) copyright the software,
and (2) offer you this license which gives you legal permission to copy,
distribute and/or modify the software.

   Also, for each author's protection and ours, we want to make certain
that everyone understands that there is no warranty for this free
software.  If the software is modified by someone else and passed on, we
want its recipients to know that what they have is not the original, so
that any problems introduced by others will not reflect on the original
authors' reputations.

   Finally, any free program is threatened constantly by software
patents.  We wish to avoid the danger that redistributors of a free
program will individually obtain patent licenses, in effect making the
program proprietary.  To prevent this, we have made it clear that any
patent must be licensed for everyone's free use or not licensed at all.

   The precise terms and conditions for copying, distribution and
modification follow.

    TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. This License applies to any program or other work which contains a
     notice placed by the copyright holder saying it may be distributed
     under the terms of this General Public License.  The "Program",
     below, refers to any such program or work, and a "work based on
     the Program" means either the Program or any derivative work under
     copyright law: that is to say, a work containing the Program or a
     portion of it, either verbatim or with modifications and/or
     translated into another language.  (Hereinafter, translation is
     included without limitation in the term "modification".)  Each
     licensee is addressed as "you".

     Activities other than copying, distribution and modification are
     not covered by this License; they are outside its scope.  The act
     of running the Program is not restricted, and the output from the
     Program is covered only if its contents constitute a work based on
     the Program (independent of having been made by running the
     Program).  Whether that is true depends on what the Program does.

  1. You may copy and distribute verbatim copies of the Program's
     source code as you receive it, in any medium, provided that you
     conspicuously and appropriately publish on each copy an appropriate
     copyright notice and disclaimer of warranty; keep intact all the
     notices that refer to this License and to the absence of any
     warranty; and give any other recipients of the Program a copy of
     this License along with the Program.

     You may charge a fee for the physical act of transferring a copy,
     and you may at your option offer warranty protection in exchange
     for a fee.

  2. You may modify your copy or copies of the Program or any portion
     of it, thus forming a work based on the Program, and copy and
     distribute such modifications or work under the terms of Section 1
     above, provided that you also meet all of these conditions:

       a. You must cause the modified files to carry prominent notices
          stating that you changed the files and the date of any change.

       b. You must cause any work that you distribute or publish, that
          in whole or in part contains or is derived from the Program
          or any part thereof, to be licensed as a whole at no charge
          to all third parties under the terms of this License.

       c. If the modified program normally reads commands interactively
          when run, you must cause it, when started running for such
          interactive use in the most ordinary way, to print or display
          an announcement including an appropriate copyright notice and
          a notice that there is no warranty (or else, saying that you
          provide a warranty) and that users may redistribute the
          program under these conditions, and telling the user how to
          view a copy of this License.  (Exception: if the Program
          itself is interactive but does not normally print such an
          announcement, your work based on the Program is not required
          to print an announcement.)

     These requirements apply to the modified work as a whole.  If
     identifiable sections of that work are not derived from the
     Program, and can be reasonably considered independent and separate
     works in themselves, then this License, and its terms, do not
     apply to those sections when you distribute them as separate
     works.  But when you distribute the same sections as part of a
     whole which is a work based on the Program, the distribution of
     the whole must be on the terms of this License, whose permissions
     for other licensees extend to the entire whole, and thus to each
     and every part regardless of who wrote it.

     Thus, it is not the intent of this section to claim rights or
     contest your rights to work written entirely by you; rather, the
     intent is to exercise the right to control the distribution of
     derivative or collective works based on the Program.

     In addition, mere aggregation of another work not based on the
     Program with the Program (or with a work based on the Program) on
     a volume of a storage or distribution medium does not bring the
     other work under the scope of this License.

  3. You may copy and distribute the Program (or a work based on it,
     under Section 2) in object code or executable form under the terms
     of Sections 1 and 2 above provided that you also do one of the
     following:

       a. Accompany it with the complete corresponding machine-readable
          source code, which must be distributed under the terms of
          Sections 1 and 2 above on a medium customarily used for
          software interchange; or,

       b. Accompany it with a written offer, valid for at least three
          years, to give any third party, for a charge no more than your
          cost of physically performing source distribution, a complete
          machine-readable copy of the corresponding source code, to be
          distributed under the terms of Sections 1 and 2 above on a
          medium customarily used for software interchange; or,

       c. Accompany it with the information you received as to the offer
          to distribute corresponding source code.  (This alternative is
          allowed only for noncommercial distribution and only if you
          received the program in object code or executable form with
          such an offer, in accord with Subsection b above.)

     The source code for a work means the preferred form of the work for
     making modifications to it.  For an executable work, complete
     source code means all the source code for all modules it contains,
     plus any associated interface definition files, plus the scripts
     used to control compilation and installation of the executable.
     However, as a special exception, the source code distributed need
     not include anything that is normally distributed (in either
     source or binary form) with the major components (compiler,
     kernel, and so on) of the operating system on which the executable
     runs, unless that component itself accompanies the executable.

     If distribution of executable or object code is made by offering
     access to copy from a designated place, then offering equivalent
     access to copy the source code from the same place counts as
     distribution of the source code, even though third parties are not
     compelled to copy the source along with the object code.

  4. You may not copy, modify, sublicense, or distribute the Program
     except as expressly provided under this License.  Any attempt
     otherwise to copy, modify, sublicense or distribute the Program is
     void, and will automatically terminate your rights under this
     License.  However, parties who have received copies, or rights,
     from you under this License will not have their licenses
     terminated so long as such parties remain in full compliance.

  5. You are not required to accept this License, since you have not
     signed it.  However, nothing else grants you permission to modify
     or distribute the Program or its derivative works.  These actions
     are prohibited by law if you do not accept this License.
     Therefore, by modifying or distributing the Program (or any work
     based on the Program), you indicate your acceptance of this
     License to do so, and all its terms and conditions for copying,
     distributing or modifying the Program or works based on it.

  6. Each time you redistribute the Program (or any work based on the
     Program), the recipient automatically receives a license from the
     original licensor to copy, distribute or modify the Program
     subject to these terms and conditions.  You may not impose any
     further restrictions on the recipients' exercise of the rights
     granted herein.  You are not responsible for enforcing compliance
     by third parties to this License.

  7. If, as a consequence of a court judgment or allegation of patent
     infringement or for any other reason (not limited to patent
     issues), conditions are imposed on you (whether by court order,
     agreement or otherwise) that contradict the conditions of this
     License, they do not excuse you from the conditions of this
     License.  If you cannot distribute so as to satisfy simultaneously
     your obligations under this License and any other pertinent
     obligations, then as a consequence you may not distribute the
     Program at all.  For example, if a patent license would not permit
     royalty-free redistribution of the Program by all those who
     receive copies directly or indirectly through you, then the only
     way you could satisfy both it and this License would be to refrain
     entirely from distribution of the Program.

     If any portion of this section is held invalid or unenforceable
     under any particular circumstance, the balance of the section is
     intended to apply and the section as a whole is intended to apply
     in other circumstances.

     It is not the purpose of this section to induce you to infringe any
     patents or other property right claims or to contest validity of
     any such claims; this section has the sole purpose of protecting
     the integrity of the free software distribution system, which is
     implemented by public license practices.  Many people have made
     generous contributions to the wide range of software distributed
     through that system in reliance on consistent application of that
     system; it is up to the author/donor to decide if he or she is
     willing to distribute software through any other system and a
     licensee cannot impose that choice.

     This section is intended to make thoroughly clear what is believed
     to be a consequence of the rest of this License.

  8. If the distribution and/or use of the Program is restricted in
     certain countries either by patents or by copyrighted interfaces,
     the original copyright holder who places the Program under this
     License may add an explicit geographical distribution limitation
     excluding those countries, so that distribution is permitted only
     in or among countries not thus excluded.  In such case, this
     License incorporates the limitation as if written in the body of
     this License.

  9. The Free Software Foundation may publish revised and/or new
     versions of the General Public License from time to time.  Such
     new versions will be similar in spirit to the present version, but
     may differ in detail to address new problems or concerns.

     Each version is given a distinguishing version number.  If the
     Program specifies a version number of this License which applies
     to it and "any later version", you have the option of following
     the terms and conditions either of that version or of any later
     version published by the Free Software Foundation.  If the Program
     does not specify a version number of this License, you may choose
     any version ever published by the Free Software Foundation.

 10. If you wish to incorporate parts of the Program into other free
     programs whose distribution conditions are different, write to the
     author to ask for permission.  For software which is copyrighted
     by the Free Software Foundation, write to the Free Software
     Foundation; we sometimes make exceptions for this.  Our decision
     will be guided by the two goals of preserving the free status of
     all derivatives of our free software and of promoting the sharing
     and reuse of software generally.

                                NO WARRANTY

 11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO
     WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE
     LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
     HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT
     WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT
     NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
     FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE
     QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
     PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY
     SERVICING, REPAIR OR CORRECTION.

 12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
     WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY
     MODIFY AND/OR REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE
     LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
     INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
     INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
     DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU
     OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY
     OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN
     ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

                      END OF TERMS AND CONDITIONS

*/
