	PROGRAM table_scripter
c Made available through the courtesy of NASA's Goddard Space Flight Center.
c------------------------------------------------------------------------------
c
c     This program is free software; you can redistribute it and/or
c     modify it under the terms of the GNU General Public License
c     as published by the Free Software Foundation; either version 2
c     of the License, or (at your option) any later version.
c     
c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.
c     
c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
c
c	PROGRAM: 	TABLE_SCRIPTER			COMPLETED: 01/19/93
c
c	PROGRAMMER: 	Phillip C. Brisco
c
c	VERSION:	1.8
c
c	FUNCTION:	This subroutine saves data from a table to an SQL
c			script file.
c
c			It uses the foreign command to get the command line
c			entry to determine the username, database name, and
c			tablename to process.  The following switches can
c			be entered from the command line:
c				-b	fill each line with data (best fit)
c				-c	chosen fields
c				-d	database name
c				-f	create a command file
c				-h	show a list of command line switches
c				-l	simple list output.  The file has
c					an extension of '.lis'.  Subparameter
c					j left justifies the text and
c					subparameter q uses quotes vice
c					apostrophes for delimiting strings
c				-o	suppresses command file message
c				-r	user entered remarks for output file
c				-s	script file
c				-t	tablename
c				-u	username
c				-v	suppresses non-error messages
c				-w	where clause (must be delimited by ")
c
c			To use this program, a symbol has to be set up to run
c			it (e.g. ugga :== $usr1:[brisco.tmp]table_scripter).
c
c			The symbol is executed with the switches (e.g.
c			ugga -ubrisco -dbrisco_pdb -tcurrent_build).
c
c			The -d and -t switches are the only required switches.
c			If the '-u' switch is omitted, the program
c			defaults to the current user's name.  Switches can be
c			entered in any order, but there must be no space
c			between the switch and the following name.
c
c			The -w, -c, and -r  switches are strings which must be
c			delimited by the double quote marks (e.g.
c			-w"filename = 'UGGA'" -c"This is a test").
c
c	MODIFICATIONS:
c
c1.1	Phillip C. Brisco	02/03/93	Added more error checking to
c						user input and added the -v
c						switch.
c
c1.2	Phillip C. Brisco	03/19/93	Added the -o switch.
c
c1.3	Phillip C. Brisco	05/12/93	Added the RECL = 512 option to
c						the open statement so that
c						fields > 128 in length can be
c						written out to a file.  This
c						should be made more dynamic in
c						the future.
c
c1.4	Phillip C. Brisco	05/19/93	Added test for the single quote
c						so that user entered single
c						quotes will be copied to the
c						database.
c
c1.5	Phillip C. Brisco	05/20/93	Changed the conversion from
c						integer and real to strings so
c						that the code is non-VAX
c						specific.  Also made the
c						record length of the output
c						file dynamic.
c
c1.6	Phillip C. Brisco	05/24/93	Put all system specific code
c						into subroutines starting with
c						the letter x.  Added the help,
c						comment and create command file
c						switches.  Added date/time
c						stamp info to output sql file.
c
c1.7	Phillip C. Brisco	07/15/93	Rearranged code so that an
c						output file is not created if
c						no data is retrieved.
c
c1.8	Phillip C. Brisco	11/17/93	Added ability for user to
c						select fields to be output.
c						Also added the option to list
c						the table in simple ASCII
c						format.
c------------------------------------------------------------------------------
c Declarations:
c
        EXEC SQL INCLUDE SQLCA;
	EXEC SQL INCLUDE SQLDA;
	PARAMETER (max_i_o_length = 200, max_out = 300)

	EXEC SQL BEGIN DECLARE SECTION
		INTEGER*4 i, j, k, l, k1, iicolumns_chan
		INTEGER*4 where_begin, where_end, table_begin, table_end
		INTEGER*4 num_digits, number_of_columns, record_length
		INTEGER*4 db_length, command_line_length, column_length
		INTEGER*4 fs_start, fs_end
		INTEGER*4 column_output_length, data_output_length
		INTEGER*4 total_number_of_rows
		CHARACTER*1 single_quote
		CHARACTER*4 version_num
		CHARACTER*5 the_time
		CHARACTER*9 the_date
		CHARACTER*16 dbname
		CHARACTER*32 column_name
		CHARACTER*32 select_table_name, current_user, file_name
		CHARACTER*32 com_file_name, the_user
		CHARACTER*80 err_msg, column_output
		CHARACTER*200 buffer
		CHARACTER*200 fields_selected, where_clause, script_clause
		CHARACTER*200 command_line
		CHARACTER*400 out_put, data_output

		STRUCTURE /field_types/
			INTEGER*2 null_ind(max_i_o_length)
			INTEGER*4 integer_number(max_i_o_length)
			REAL*8 real_number(max_i_o_length)
			CHARACTER*200 character_data(max_i_o_length)
		END STRUCTURE

		RECORD /field_types/b(10)
	EXEC SQL END DECLARE SECTION

	RECORD /IISQLDA/result_descriptor
	RECORD /IISQLNAME/sqlname

	INTEGER*4 list_begin, list_end, comment_begin, comment_end
	INTEGER*4 script_begin, script_end
	INTEGER*2 the_logical_length
	INTEGER*4 istat, normal_ret
	LOGICAL write_the_junque
	LOGICAL uflag, tflag, dflag, wflag, exit_flag, verbose_off, sflag
	LOGICAL command_line_off, create_command_file, best_fit
	LOGICAL user_added_text, help_me, list_format, left_justify
	LOGICAL double_quote
c------------------------------------------------------------------------------
	version_num = '1.80'
	CALL xget_command_line(command_line,command_line_length,
	1	max_i_o_length)
	current_user = ' '
	select_table_name = ' '
	dbname = ' '
	where_clause = ' '
	script_clause = ' '
	exit_flag = .FALSE.
	uflag = .FALSE.
	tflag = .FALSE.
	dflag = .FALSE.
	wflag = .FALSE.
	sflag = .FALSE.
	verbose_off = .FALSE.
	command_line_off = .FALSE.
	create_command_file = .FALSE.
	user_added_text = .FALSE.
	best_fit = .FALSE.
	list_format = .FALSE.
	left_justify = .FALSE.
	double_quote = .FALSE.
	data_output = ' '
	data_output_length = 0

c----------------------- Beginning of test for command line input -------------
	fields_selected = '*'
	fs_start = 1
	fs_end = 1

	DO i = 1,command_line_length - 1

c
c		Database owner.
c
		IF ((command_line(i:i + 1) .EQ. '-u' .OR.
	1		command_line(i:i + 1) .EQ. '-U') .AND. .NOT.
	2		uflag) THEN
			j = i

			DO WHILE (command_line(j:j) .GT. ' ' .AND. j .LE.
	1			max_i_o_length)

c
c				Lowercase the substring.
c
				IF (command_line(j:j) .GE. 'A' .AND.
	1				command_line(j:j) .LE. 'Z')
	2				command_line(j:j) = CHAR (ICHAR (
	3				command_line(j:j)) .OR. 32)
				j = j + 1
			ENDDO

			IF (j - 1 .GT. i + 2) THEN
				uflag = .TRUE.
				current_user = command_line(i + 2:j - 1)
			ENDIF

		  ELSE IF ((command_line(i:i + 1) .EQ. '-u' .OR.
	1		command_line(i:i + 1) .EQ. '-U') .AND. uflag) THEN
			WRITE (*,*) 'Multiple users specified in command line'
			exit_flag = .TRUE.
		ENDIF

c
c		Database table.
c
		IF (command_line(i:i + 1) .EQ. '-t' .OR.
	1		command_line(i:i + 1) .EQ. '-T' .AND. .NOT.
	2		tflag) THEN
			j = i + 2

			DO WHILE (command_line(j:j) .LE. ' ' .AND. j .LE.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			table_begin = j

			DO WHILE (command_line(j:j) .GT. ' ' .AND. j .LE.
	1			max_i_o_length)
c
c				Lowercase the substring.
c
				IF (command_line(j:j) .GE. 'A' .AND.
	1				command_line(j:j) .LE. 'Z')
	2				command_line(j:j) = CHAR (ICHAR (
	3				command_line(j:j)) .OR. 32)
				j = j + 1
			ENDDO

			IF (j - 1 .GT. table_begin) THEN
				tflag = .TRUE.
				select_table_name = command_line(table_begin:
	1				j - 1)
				table_end = j - i
			ENDIF

		  ELSE IF (command_line(i:i + 1) .EQ. '-t' .OR.
	1		command_line(i:i + 1) .EQ. '-T' .AND. tflag) THEN
			WRITE (*,*) 'Multiple tables specified in command line'
			exit_flag = .TRUE.
		ENDIF

c
c		Database name.
c
		IF (command_line(i:i + 1) .EQ. '-d' .OR.
	1		command_line(i:i + 1) .EQ. '-D' .AND. .NOT.
	2		dflag) THEN
			j = i

			DO WHILE (command_line(j:j) .GT. ' ' .AND. j .LE.
	1			max_i_o_length)
c
c				Lowercase the substring.
c
				IF (command_line(j:j) .GE. 'A' .AND.
	1				command_line(j:j) .LE. 'Z')
	2				command_line(j:j) = CHAR (ICHAR (
	3				command_line(j:j)) .OR. 32)
				j = j + 1
			ENDDO

			IF (j - 1 .GT. i + 2) THEN
				dbname = command_line(i + 2:j - 1)
				dflag = .TRUE.
			ENDIF

		  ELSE IF (command_line(i:i + 1) .EQ. '-d' .OR.
	1		command_line(i:i + 1) .EQ. '-D' .AND. dflag) THEN
			WRITE (*,*) 'Multiple databases specified on command'//
	1			' line'
			exit_flag = .TRUE.
		ENDIF

c
c		Where clause.
c
		IF (command_line(i:i + 1) .EQ. '-w' .OR.
	1		command_line(i:i + 1) .EQ. '-W' .AND. .NOT.
	2		wflag) THEN
			wflag = .TRUE.
			j = i

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LE.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			j = j + 1
			where_begin = j

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LT.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			IF (j .EQ. max_i_o_length .AND.
	1			command_line(j:j) .NE. '"') THEN
				where_end = where_begin
			  ELSE
				where_end = j - 1
			ENDIF

			IF (where_end .GT. where_begin) THEN
				where_clause = 'WHERE '//command_line(
	1				where_begin:where_end)
			  ELSE
				WRITE (*,*) 'Illegal where clause'
				exit_flag = .TRUE.
			ENDIF

		  ELSE IF (command_line(i:i + 1) .EQ. '-w' .OR.
	1		command_line(i:i + 1) .EQ. '-W' .AND. wflag) THEN
			WRITE (*,*), 'Multiple where clauses specified on '//
	2			'command line'
			exit_flag = .TRUE.
		ENDIF

c
c		Script file.
c
		IF (command_line(i:i + 1) .EQ. '-s' .OR.
	1		command_line(i:i + 1) .EQ. '-S' .AND. .NOT.
	2		sflag) THEN
			j = i + 2

			DO WHILE (command_line(j:j) .LE. ' ' .AND. j .LE.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			script_begin = j

			DO WHILE (command_line(j:j) .GT. ' ' .AND. j .LE.
	1			max_i_o_length)
c
c				Lowercase the substring.
c
				IF (command_line(j:j) .GE. 'A' .AND.
	1				command_line(j:j) .LE. 'Z')
	2				command_line(j:j) = CHAR (ICHAR (
	3				command_line(j:j)) .OR. 32)
				j = j + 1
			ENDDO

			IF (j - 1 .GT. i) THEN
				sflag = .TRUE.
				script_clause = command_line(script_begin:
	1				j - 1)
				script_end = j - i
			  ELSE
				WRITE (*,*) 'Illegal script file'
				exit_flag = .TRUE.
			ENDIF

		  ELSE IF (command_line(i:i + 1) .EQ. '-s' .OR.
	1		command_line(i:i + 1) .EQ. '-S' .AND. sflag) THEN
			WRITE (*,*), 'Multiple script files specified on '//
	2			'command line'
			exit_flag = .TRUE.
		ENDIF

c
c		Verbose switch
c
		IF (command_line(i:i + 1) .EQ. '-v' .OR.
	1		command_line(i:i + 1) .EQ. '-V') THEN
			verbose_off = .TRUE.
		ENDIF

c
c		Simple list switch
c
		IF (command_line(i:i + 1) .EQ. '-l' .OR.
	1		command_line(i:i + 1) .EQ. '-L') THEN
			list_format = .TRUE.
			j = i

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LE.
	1			max_i_o_length)
				IF (command_line(j:j) .EQ. 'j' .OR.
	1				command_line(j:j) .EQ. 'J')
	2				left_justify = .TRUE.
				IF (command_line(j:j) .EQ. 'q' .OR.
	1				command_line(j:j) .EQ. 'Q')
	2				double_quote = .TRUE.
				j = j + 1
			ENDDO

			j = j + 1
			list_begin = j

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LT.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			IF (j .EQ. max_i_o_length .AND.
	1			command_line(j:j) .NE. '"') THEN
				list_end = list_begin - 1
			  ELSE
				list_end = j - 1
			ENDIF

		ENDIF

c
c		Command file output switch.
c
		IF (command_line(i:i + 1) .EQ. '-o' .OR.
	1		command_line(i:i + 1) .EQ. '-O') THEN
			command_line_off = .TRUE.
		ENDIF

c
c		Create command file switch.
c
		IF (command_line(i:i + 1) .EQ. '-f' .OR.
	1		command_line(i:i + 1) .EQ. '-F') THEN
			create_command_file = .TRUE.
		ENDIF

c
c		Add user-entered comment switch.
c
		IF (command_line(i:i + 1) .EQ. '-r' .OR.
	1		command_line(i:i + 1) .EQ. '-R') THEN
			user_added_text = .TRUE.
			j = i

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LE.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			j = j + 1
			comment_begin = j

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LT.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			IF (j .EQ. max_i_o_length .AND.
	1			command_line(j:j) .NE. '"') THEN
				comment_end = comment_begin - 1
			  ELSE
				comment_end = j - 1
			ENDIF

			IF (comment_end .LT. comment_begin) THEN
				WRITE (*,*) 'Ignoring invalid user comment...'
				user_added_text = .FALSE.
			ENDIF

		ENDIF

c
c		Fields selected.
c
		IF (command_line(i:i + 1) .EQ. '-c' .OR.
	1		command_line(i:i + 1) .EQ. '-C') THEN
			j = i

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LE.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			j = j + 1
			fs_start = j

			DO WHILE (command_line(j:j) .NE. '"' .AND. j .LT.
	1			max_i_o_length)
				j = j + 1
			ENDDO

			IF (j .EQ. max_i_o_length .AND.
	1			command_line(j:j) .NE. '"') THEN
				fs_end = fs_start - 1
			  ELSE
				fs_end = j - 1
			ENDIF

			IF (fs_end .LT. fs_start) THEN
				WRITE (*,*) 'Ignoring invalid user selected '//
	1				'fields...'
				fields_selected = '*'
				fs_start = 1
				fs_end = 1
			  ELSE
				fields_selected = command_line(fs_start:
	1				fs_end)
				fs_end = fs_end - fs_start + 1
				fs_start = 1
			ENDIF

		ENDIF

c
c		Best fit switch.
c
		IF (command_line(i:i + 1) .EQ. '-b' .OR.
	1		command_line(i:i + 1) .EQ. '-B') THEN
			best_fit = .TRUE.
		ENDIF

c
c		Help switch.
c
		IF (command_line(i:i + 1) .EQ. '-h' .OR.
	1		command_line(i:i + 1) .EQ. '-H') THEN
			help_me = .TRUE.
		ENDIF

	ENDDO
c---------------------- End of testing for command line input -----------------

	IF (help_me) THEN
		WRITE (*,*), ' '
		WRITE (*,*), ' '
		WRITE (*,*), 'The following are legal command line switches:'
		WRITE (*,*), ' '
		WRITE (*,*), '-b	fill each line with data (best fit)'
		WRITE (*,*), '-c	user-selected data fields'
		WRITE (*,*), '-d	database name'
		WRITE (*,*), '-f	create a command file'
		WRITE (*,*), '-h	show a list of command line switches'
		WRITE (*,*), '-l	create a simple list file.  This '//
	1		'has an extension of ".lis"'
		WRITE (*,*), '	Subparameter j left justifies the text and '//
	1		'subparameter q uses quotes'
		WRITE (*,*), '	vice apostrophes for delimiting strings.'
		WRITE (*,*), '-o	suppresses command file message'
		WRITE (*,*), '-r	user entered comment for output file'
		WRITE (*,*), '-s	name of script file'
		WRITE (*,*), '-t	tablename'
		WRITE (*,*), '-u	username'
		WRITE (*,*), '-v	suppresses non-error messages'
		WRITE (*,*), '-w	where clause (must be delimited by ")'
		WRITE (*,*), ' '
		CALL EXIT
	ENDIF

	where_end = where_end - where_begin + 7
	where_begin = 1
	single_quote = CHAR(39)
	CALL xget_jpi(the_user,istat,normal_ret)

c
c	Lowercase the username since searches are case sensitive in INGRES.
c
	DO i = 1,32
		IF (ICHAR(the_user(i:i)) .GE. 65 .AND. ICHAR(the_user(i:i))
	1		.LE. 90) the_user(i:i) = CHAR(ICHAR(the_user(i:i))
	2		.OR. 32)
	ENDDO

c
c	IF the user did not select the user flag when entering the command
c	line, use the account name as the user.
c
	IF (current_user(1:1) .LE. ' ') THEN
		CALL xget_jpi(current_user,istat,normal_ret)

		IF (istat .NE. normal_ret) THEN
			current_user = ' '
			WRITE (*,*) 'User information not entered.'
		  ELSE

c
c			Lowercase the username since searches are case
c			sensitive in INGRES.
c
			DO i = 1,32
				IF (ICHAR(current_user(i:i)) .GE. 65 .AND.
	1				ICHAR(current_user(i:i)) .LE. 90)
	2				current_user(i:i) = CHAR(ICHAR(
	3				current_user(i:i)) .OR. 32)
			ENDDO

			uflag = .TRUE.
		ENDIF

	ENDIF

c
c	IF the user name, table name, or database name is not found, exit the
c	program, else get the lengths of the aforementioned names.
c
	IF (.NOT. uflag .OR. .NOT. tflag .OR. .NOT. dflag .OR. exit_flag) THEN
		IF (.NOT. uflag) WRITE (*,*) 'User information not found.'
		IF (.NOT. tflag) WRITE (*,*) 'Table information not entered.'
		IF (.NOT. dflag) WRITE (*,*) 'Database information not '//
	1		'entered.'
		CALL EXIT
	  ELSE

		DO i = 1,16
			IF (dbname(i:i) .GT. ' ') db_length = i
		ENDDO

		DO i = 1,32
			IF (current_user(i:i) .GT. ' ') the_logical_length = i
			IF (select_table_name(i:i) .GT. ' ') j = i
		ENDDO

	ENDIF

	IF (.NOT. verbose_off) WRITE (*,*) 'Starting table_scripter...'

	EXEC SQL CONNECT :dbname IDENTIFIED BY :current_user;

	IF (sqlcod .NE. 0) THEN
		EXEC SQL INQUIRE_INGRES (:err_msg = ERRORTEXT)
		j = 0

		DO i = 1,80
			IF (ichar(err_msg(i:i)) .EQ. 46) j = i
		ENDDO

		WRITE (*,*) err_msg(1:j)
		CALL EXIT
	ENDIF

	IF (.NOT. verbose_off) THEN
		WRITE (*,*), ' '
		WRITE (*,*), 'Creating data and command files for table '//
	1		select_table_name(1:j)
		WRITE (*,*), 'on the '//dbname(1:db_length)//' database '//
	1		'owned by '//current_user(1:the_logical_length)//' ...'
	ENDIF

	i = 0

	IF (list_format) THEN

		IF (sflag) THEN
			file_name = script_clause(:script_end)
		  ELSE
			EXEC SQL SELECT concat(squeeze(:select_table_name),
	1			'.lis') INTO :file_name
		ENDIF

	  ELSE

		IF (sflag) THEN
			file_name = script_clause(:script_end)
		  ELSE
			EXEC SQL SELECT concat(squeeze(:select_table_name),
	1			'.sql') INTO :file_name
		ENDIF

	ENDIF

	j = 0
	k = 0

	DO i=1,32
		IF (file_name(i:i) .GT. ' ') j = i
		IF (file_name(i:i) .EQ. '.') k = i
	ENDDO

	IF (k .EQ. 0) file_name(j + 1:) = '.dat'
	WRITE (*,*) 'The output file name is ',file_name

	record_length = 0
	buffer = 'SELECT '//fields_selected(fs_start:fs_end)//' FROM '//
	1	select_table_name(1:table_end)//' '//
	2	where_clause(where_begin:where_end)//' '
	CALL dynamic_open (b(1), result_descriptor, sqlname,
	1	record_length, number_of_columns, buffer, s1, c1,
	2	current_user, select_table_name)

	EXEC SQL DECLARE c1 CURSOR FOR s1;
	EXEC SQL OPEN c1;
	EXEC SQL FETCH c1 USING DESCRIPTOR :result_descriptor;

	IF (sqlcod .NE. 0) THEN
		EXEC SQL CLOSE c1;
		sqlcod = -1
		WRITE (*,*) 'Output file not created due to the lack '//
	1		'of data.'
	ENDIF

	IF (sqlcod .NE. 0) RETURN
	EXEC SQL CLOSE c1;
	EXEC SQL OPEN c1;
	CALL xget_lun(iicolumns_chan)

c
c	Create the SQL data file.
c
	OPEN (iicolumns_chan, FILE=file_name, STATUS='new',
	1	RECL = record_length, CARRIAGECONTROL = 'list')


	write_the_junque = .TRUE.
	EXEC SQL SELECT squeeze(_date(int4(dbmsinfo('_bintim')))),
	1	squeeze(_time(int4(dbmsinfo('_bintim'))))
	2	INTO :the_date, :the_time

	WRITE (iicolumns_chan,80) '/* Table_scripter '//version_num//
	1	' was used to copy data from: '
	WRITE (iicolumns_chan,80) '/*'
	WRITE (iicolumns_chan,80) '/*	Table:    '//select_table_name
	WRITE (iicolumns_chan,80) '/*	Database: '//dbname
	WRITE (iicolumns_chan,80) '/*	Owner:    '//current_user
	WRITE (iicolumns_chan,80) '/*	User:     '//the_user

	IF (best_fit) then
		WRITE (iicolumns_chan,80) '/*	Format:   best fit'
	  ELSE
		WRITE (iicolumns_chan,80) '/*	Format:   normal'
	ENDIF

	IF (list_format) THEN
		WRITE (iicolumns_chan,80) '/*	Style:    simple list file'
	  ELSE
		WRITE (iicolumns_chan,80) '/*	Style:    SQL script file'
	ENDIF

	IF (create_command_file) WRITE (iicolumns_chan,80)
	1	'/*	associated command file created'
	WRITE (iicolumns_chan,80) '/*'
	WRITE (iicolumns_chan,80) '/* On '//the_date//' at '//the_time
	WRITE (iicolumns_chan,80) '/*'

	IF (user_added_text) THEN
		WRITE (iicolumns_chan,80) '/* '//command_line(comment_begin:
	1		comment_end)
	ENDIF

	WRITE (iicolumns_chan,80) '/**/'
	total_number_of_rows = 0

	DO WHILE (write_the_junque)
		i = 1

		DO WHILE (i .LE. max_i_o_length)
			b(1).null_ind(i) = 0
			j = 1

			DO WHILE (j .LE. max_i_o_length)
				b(1).character_data(i)(j:j) = ' '
				j = j + 1
			ENDDO

			out_put(i:i) = ' '
			i = i + 1
		ENDDO

		EXEC SQL FETCH c1 USING DESCRIPTOR :result_descriptor;

c
c	SQLCOD is equal to zero only if FETCH was completed successfully.
c
		IF (sqlcod .EQ. 0) THEN
			write_the_junque = .TRUE.
			total_number_of_rows = total_number_of_rows + 1
		  ELSE
			write_the_junque = .FALSE.
			GOTO 100
		ENDIF

c
c		We have to test for blank spaces.  If a character variable
c		only contains blanks spaces, we assume that it is null.
c
		DO j = 1,result_descriptor.sqld

			IF (result_descriptor.sqlvar(j).sqltype .EQ. -20) THEN
				b(1).null_ind(j) = -1

				DO k = 1,max_i_o_length
					IF (b(1).character_data(j)(k:k)
	1					.GT. ' ') b(1).null_ind(j) = 0
				ENDDO

			ENDIF

		ENDDO

		k = 0

c
c		Adjust the sqld field of the result_descriptor structure so
c		that any fields with no data at the end are not even processed.
c
		DO j = 1,result_descriptor.sqld
			if (b(1).null_ind(j) .EQ. 0) k = j
		ENDDO

		result_descriptor.sqld = k

		IF (list_format) GOTO 1000

		WRITE (iicolumns_chan,10) select_table_name

c
c		Write the names of the columns to the SQL file.
c
		DO j = 1,result_descriptor.sqld

			IF (b(1).null_ind(j) .EQ. 0 .OR. result_descriptor.
	1			sqlvar(j).sqltype .GT. 0) THEN
			column_name = result_descriptor.sqlvar(j).sqlname.
	1			sqlnamec
			k = 1

			DO WHILE (ICHAR(column_name(k:k)) .LE. 32 .AND.
	1			k .LE. max_i_o_length)
				k = k + 1
			ENDDO

			l = k

			DO WHILE (ICHAR(column_name(l:l)) .GT. 32 .AND.
	1			l .LE. max_i_o_length)
				l = l + 1
			ENDDO

			column_name(1:l - k) = column_name(k:l)
			k = (l - k) + 1

			IF (j .EQ. 1 .AND. j .EQ. result_descriptor.sqld) THEN
				column_name(k:k) = ')'
			  ELSE IF (j .NE. result_descriptor.sqld) THEN
				column_name(k:k) = ','
			  ELSE
				column_name(k:k) = ')'
			ENDIF

			IF (best_fit) THEN

				IF (column_output_length + k .LE. 70) THEN
					column_output = column_output(1:
	1					column_output_length)//
	2					column_name(1:k)
					column_output_length = k +
	1					column_output_length
				  ELSE
					WRITE (iicolumns_chan,20) column_output
	1					(1:column_output_length)
					column_output = column_name(1:k)
					column_output_length = k
				ENDIF

			  ELSE
				WRITE (iicolumns_chan,20) column_name
			ENDIF

			ENDIF
		ENDDO

		IF (best_fit) WRITE (iicolumns_chan,20) column_output(1:
	1		column_output_length)
		column_output_length = 0
		WRITE (iicolumns_chan,40) ' '

1000		CONTINUE
c
c		Write the column data to the SQL file.
c
		DO j = 1,result_descriptor.sqld
			IF (b(1).null_ind(j) .EQ. 0 .OR. result_descriptor.
	1			sqlvar(j).sqltype .GT. 0) THEN

			IF (ABS(result_descriptor.sqlvar(j).sqltype) .EQ. 30
	1			.OR. (ABS(result_descriptor.sqlvar(j).sqltype)
	2			.EQ. 31) .OR. (ABS(result_descriptor.sqlvar(
	3			j).sqltype) .EQ. 5)) THEN

				IF (ABS(result_descriptor.sqlvar(j).sqltype)
	1				.EQ. 30) THEN
					WRITE (out_put, '(I)'),
	1					b(1).integer_number(j)
				  ELSE
					WRITE (out_put, '(G)'),
	1					b(1).real_number(j)
					num_digits = 0

					DO i = 1,20
						IF (ICHAR(out_put(i:i)) .GT.
	1						48) num_digits = i
					ENDDO

					IF (num_digits .EQ. 0) THEN
						out_put = '0.0'
						num_digits = 3
					  ELSE
						out_put = out_put(1:num_digits)
					ENDIF

				ENDIF

c
c				Strip off leading blanks.
c
				k = 1

				DO WHILE (ICHAR(out_put(k:k))
	1				.LE. 32 .AND. k .LE.
	2				max_i_o_length)
					k = k + 1
				ENDDO

c
c				Strip off trailing blanks.
c
				l = k

				DO WHILE (ICHAR(out_put(l:l))
	1				.GT. 32 .AND. l .LE.
	2				max_i_o_length)
					l = l + 1
				ENDDO

				out_put(1:l - k) = out_put(k:l)
				k = (l - k) + 1

				IF (list_format) THEN
					out_put(k:k) = ' '
				  ELSE
					out_put(k:k) = ','
				ENDIF

			  ELSE IF (ABS(result_descriptor.sqlvar(j).sqltype)
	1			.EQ. 20) THEN

				IF (double_quote) THEN
					out_put = '"'//b(1).character_data(j)
				  ELSE
					out_put = single_quote//
	1					b(1).character_data(j)
				ENDIF

				k = 2
				k1 = k

				DO WHILE (k1 .LT. max_i_o_length)
					IF (ICHAR(out_put(k1:k1)) .GT. 32)
	1					k = k1
					IF (ICHAR(out_put(k1:k1)) .EQ. 39) THEN

						IF (double_quote) THEN
							out_put = out_put(
	1							1:k1)//'"'//
	2							out_put(k1+1:)
						  ELSE
							out_put = out_put
	1							(1:k1)//
	2							single_quote//
	3							out_put(k1+1:)
						ENDIF

						k1 = k1 + 1
					ENDIF

					k1 = k1 + 1
				ENDDO

				k = k + 1

c
c	For dates which are put into character columns, the following format
c	is necessary to get the entire date.
c
				IF (out_put(5:7) .EQ. 'JAN' .OR.
	1				out_put(5:7) .EQ. 'FEB' .OR.
	2				out_put(5:7) .EQ. 'MAR' .OR.
	3				out_put(5:7) .EQ. 'APR' .OR.
	4				out_put(5:7) .EQ. 'MAY' .OR.
	5				out_put(5:7) .EQ. 'JUN' .OR.
	6				out_put(5:7) .EQ. 'JUL' .OR.
	7				out_put(5:7) .EQ. 'AUG' .OR.
	8				out_put(5:7) .EQ. 'SEP' .OR.
	9				out_put(5:7) .EQ. 'OCT' .OR.
	1				out_put(5:7) .EQ. 'NOV' .OR.
	2				out_put(5:7) .EQ. 'DEC') k = 22

				IF (out_put(5:7) .EQ. 'jan' .OR.
	1				out_put(5:7) .EQ. 'feb' .OR.
	2				out_put(5:7) .EQ. 'mar' .OR.
	3				out_put(5:7) .EQ. 'apr' .OR.
	4				out_put(5:7) .EQ. 'may' .OR.
	5				out_put(5:7) .EQ. 'jun' .OR.
	6				out_put(5:7) .EQ. 'jul' .OR.
	7				out_put(5:7) .EQ. 'aug' .OR.
	8				out_put(5:7) .EQ. 'sep' .OR.
	9				out_put(5:7) .EQ. 'oct' .OR.
	1				out_put(5:7) .EQ. 'nov' .OR.
	2				out_put(5:7) .EQ. 'dec') k = 22

			IF (ABS(result_descriptor.sqlvar(j).sqltype) .EQ. 3)
	1			k = 25

				IF (list_format) THEN
					k = k + 1

					IF (double_quote) THEN
						out_put(k - 1:k) = '" '
					  ELSE
						out_put(k - 1:k) =
	1						single_quote//' '
					ENDIF

				  ELSE
					k = k + 1
					out_put(k - 1:k) = single_quote//','
				ENDIF

			ENDIF

			IF (best_fit) THEN

				IF (data_output_length + k .LE. 70) THEN
					data_output = data_output(:
	1					data_output_length)//
	2					out_put(1:k)
					data_output_length = k +
	1					data_output_length
				  ELSE
					WRITE (iicolumns_chan,20)
	1					data_output(:
	2					data_output_length)
					data_output = out_put(1:k)
					data_output_length = k
				ENDIF

			  ELSE

				IF (j .LT. result_descriptor.sqld .OR.
	1				list_format) THEN

					IF (left_justify) THEN
						WRITE (iicolumns_chan,80)
	1						out_put(1:k)
					  ELSE
						WRITE (iicolumns_chan,20)
	1						out_put(1:k)
					ENDIF

				  ELSE
					out_put(k:k) = ')'
					WRITE (iicolumns_chan,50) out_put(1:k)
				ENDIF

			ENDIF

			ENDIF
		ENDDO

		IF (best_fit) THEN

			DO i = 1,max_out
				IF (ICHAR(data_output(i:i)) .GT. 32) j = i
			ENDDO

			IF (.NOT. list_format) THEN
				data_output(j:j) = ')'
				WRITE (iicolumns_chan,50) data_output(1:
	1				data_output_length)
			  ELSE

				IF (left_justify) THEN
					WRITE (iicolumns_chan,80) data_output
	1					(1:data_output_length)
				  ELSE
					WRITE (iicolumns_chan,20) data_output
	1					(1:data_output_length)
				ENDIF

			ENDIF

		ENDIF

		data_output_length = 0
		data_output = ' '

		IF (list_format .AND. list_end .GE. list_begin)
	1		WRITE (iicolumns_chan,80)
	2		command_line(list_begin:list_end)
	ENDDO

100	IF (.NOT. list_format) THEN
		WRITE (iicolumns_chan,90) '/* Total number of rows = ',
	1		total_number_of_rows
		WRITE (iicolumns_chan,80) '*/'
	ENDIF

	CLOSE (iicolumns_chan)

	IF (create_command_file) THEN
		EXEC SQL SELECT concat(squeeze(:select_table_name), '.com')
	1		INTO :com_file_name

c
c		Create the command file.
c
		OPEN (iicolumns_chan, FILE=com_file_name, STATUS='new',
	1		CARRIAGECONTROL = 'list')

		DO i = 1,32
			IF (ICHAR(file_name(i:i)) .GT. 32) j = i
			IF (ICHAR(current_user(i:i)) .GT. 32) k = i
		ENDDO

		DO i = 1,16
			IF (ICHAR(dbname(i:i)) .GT. 32) l = i
		ENDDO

		IF (.NOT. command_line_off) WRITE (iicolumns_chan,60)
	1		select_table_name
		WRITE (iicolumns_chan,70) file_name(1:j), dbname(1:l),
	1		current_user(1:k)

		CLOSE (iicolumns_chan)
	ENDIF

10	FORMAT ('INSERT INTO ', A, '(')
20	FORMAT (8x, A)
30	FORMAT (8x, A, ')')
40	FORMAT (2x, 'VALUES (', A)
50	FORMAT (8x, A, /, '\g\r')
60	FORMAT ('$ WRITE SYS$OUTPUT "Reloading table ', A, '"')
70	FORMAT ('$ SQL <', A ' > ', A, ' -u',A)
80	FORMAT (A)
90	FORMAT (A,I)

	END

c
c	This subroutine gets user-entered switch information from the
c	command line which fires off the image.
c
	SUBROUTINE xget_command_line (command_line,command_line_length,
	1	max_i_o_length)

	INCLUDE '($ssdef)'
	INTEGER*4 istat, command_line_length, max_i_o_length
	CHARACTER*(*) command_line

	istat = LIB$GET_FOREIGN (command_line,,command_line_length,)
	IF (command_line_length .GT. max_i_o_length)
	1	command_line_length = max_i_o_length

	RETURN

	END

c
c	Get the account name.
c
	SUBROUTINE XGET_JPI (current_user,istat,normal_ret)

	INCLUDE '($jpidef)'
	INCLUDE '($ssdef)'
	INTEGER*4 istat, normal_ret, LIB$GETJPI
	CHARACTER*(*) current_user

	normal_ret = ss$_normal
	istat = LIB$GETJPI(%REF(jpi$_username),,,,%DESCR(current_user))

	RETURN

	END

c
c	Get a free logical unit number.
c
	SUBROUTINE xget_lun(lun)

	INCLUDE '($ssdef)'
	INTEGER*4 iicolumns_chan

	istat = LIB$GET_LUN (lun)

	RETURN

	END

c
c	Sets up dynamism so that we can get data.
c
	SUBROUTINE dynamic_open (bb, results, sqlname,
	1	record_length, number_of_columns, buffer, s1, c1,
	2	current_user, select_table_name)

	EXEC SQL INCLUDE SQLDA
	EXEC SQL INCLUDE SQLCA
	PARAMETER (max_i_o_length = 200)

	EXEC SQL BEGIN DECLARE SECTION
		CHARACTER*(*) current_user, select_table_name

		INTEGER*4 i, j, k, record_length, number_of_columns
		INTEGER*4 s1, c1
		CHARACTER*(*) buffer
		CHARACTER*80 err_msg

		STRUCTURE /field_type/
			INTEGER*2 null_ind(max_i_o_length)
			INTEGER*4 integer_number(max_i_o_length)
			REAL*8 real_number(max_i_o_length)
			CHARACTER*200 character_data(max_i_o_length)
		END STRUCTURE

		RECORD /field_type/bb
	EXEC SQL END DECLARE SECTION

	RECORD /IISQLDA/results
	RECORD /IISQLNAME/sqlname

c
c	Get the number of columns from the iicolumns table.
c
	EXEC SQL SELECT COUNT (*)
	1	INTO	:i
	2	FROM	iicolumns
	3	WHERE	table_owner = :current_user
	4	AND	table_name = :select_table_name

	number_of_columns = i
	results.sqln = number_of_columns

c
c	Whenever not found then continue (sqlcod = 100 if not found) 
c
	EXEC SQL WHENEVER NOT FOUND CONTINUE;

c
c	Equate the variable buffer to 's1' using the PREPARE statement, set
c	up 's1' to use the SQLDA structure through the 'results'
c	record, and do a dynamic SELECT using cursors.
c
c	If the data type is string but not CHAR, then convert it to CHAR
c	since dynamic SQL has a difficult time fitting VARCHAR onto a CHAR
c	host variable.
c
c	30/-30 = integer, 31,-31 = real, 20/-20 = char, 21/-21 = varchar,
c	3/-3 = date, 5/-5 = money.
c
	EXEC SQL PREPARE s1 FROM :buffer;

	IF (sqlcod .NE. 0) THEN
		EXEC SQL INQUIRE_INGRES (:err_msg = errortext)
		j = 0

		DO i = 1,80
			IF (ichar(err_msg(i:i)) .EQ. 46) j = i
		ENDDO

		WRITE (*,*),err_msg(1:j)
		CALL EXIT
	ENDIF

	EXEC SQL DESCRIBE s1 INTO :results;

	IF (sqlcod .NE. 0) THEN
		EXEC SQL INQUIRE_INGRES (:err_msg = errortext)
		j = 0

		DO i = 1,80
			IF (ichar(err_msg(i:i)) .EQ. 46) j = i
		ENDDO

		WRITE (*,*),err_msg(1:j)
		CALL EXIT
	ENDIF

	DO i = 1,results.sqln

		IF (results.sqlvar(i).sqltype .EQ. 30) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.integer_number(i))
			record_length = record_length + 10
		  ELSE IF (results.sqlvar(i).sqltype .EQ. -30) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.integer_number(i))
			results.sqlvar(i).sqlind =
	1			%LOC(bb.null_ind(i))
			record_length = record_length + 10
		  ELSE IF (results.sqlvar(i).sqltype .EQ. 31) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.real_number(i))
			record_length = record_length + 20
		  ELSE IF (results.sqlvar(i).sqltype .EQ. -31) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.real_number(i))
			results.sqlvar(i).sqlind =
	1			%LOC(bb.null_ind(i))
			record_length = record_length + 20
		  ELSE IF (results.sqlvar(i).sqltype .EQ. 20) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.character_data(i))
			record_length = record_length + results.sqlvar(i
	1			).sqllen
			results.sqlvar(i).sqllen =
	1			max_i_o_length
		  ELSE IF (results.sqlvar(i).sqltype .EQ. -20) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.character_data(i))
			record_length = record_length + results.sqlvar(i
	1			).sqllen
			results.sqlvar(i).sqllen =
	1			max_i_o_length
			results.sqlvar(i).sqlind =
	1			%LOC(bb.null_ind(i))
		  ELSE IF (results.sqlvar(i).sqltype .EQ. 21) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.character_data(i))
			record_length = record_length + results.sqlvar(i
	1			).sqllen
			results.sqlvar(i).sqltype = 20
			results.sqlvar(i).sqllen =
	1			max_i_o_length
		  ELSE IF (results.sqlvar(i).sqltype .EQ. -21) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.character_data(i))
			record_length = record_length + results.sqlvar(i
	1			).sqllen
			results.sqlvar(i).sqltype = -20
			results.sqlvar(i).sqllen =
	1			max_i_o_length
			results.sqlvar(i).sqlind =
	1			%LOC(bb.null_ind(i))
		  ELSE IF (results.sqlvar(i).sqltype .EQ. 3) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.character_data(i))
			record_length = record_length + results.sqlvar(i
	1			).sqllen
			results.sqlvar(i).sqltype = 20
			results.sqlvar(i).sqllen =
	1			max_i_o_length
		  ELSE IF (results.sqlvar(i).sqltype .EQ. -3) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.character_data(i))
			record_length = record_length + results.sqlvar(i
	1			).sqllen
			results.sqlvar(i).sqltype = -20
			results.sqlvar(i).sqllen =
	1			max_i_o_length
			results.sqlvar(i).sqlind =
	1			%LOC(bb.null_ind(i))
		  ELSE IF (results.sqlvar(i).sqltype .EQ. 5) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.real_number(i))
			record_length = record_length + 20
		  ELSE IF (results.sqlvar(i).sqltype .EQ. -5) THEN
			results.sqlvar(i).sqldata =
	1			%LOC(bb.real_number(i))
			results.sqlvar(i).sqlind =
	1			%LOC(bb.null_ind(i))
			record_length = record_length + 20
		ENDIF

	ENDDO

	IF (record_length .LT. 128) record_length = 128

	RETURN

	END


c  The GNU General Public License is as follows:
c
c                         Version 2, June 1991
c
c     Copyright (C) 1989, 1991 Free Software Foundation, Inc.
c     675 Mass Ave, Cambridge, MA 02139, USA
c     
c     Everyone is permitted to copy and distribute verbatim copies
c     of this license document, but changing it is not allowed.
c
cPreamble
c========
c
c   The licenses for most software are designed to take away your
cfreedom to share and change it.  By contrast, the GNU General Public
cLicense is intended to guarantee your freedom to share and change free
csoftware--to make sure the software is free for all its users.  This
cGeneral Public License applies to most of the Free Software
cFoundation's software and to any other program whose authors commit to
cusing it.  (Some other Free Software Foundation software is covered by
cthe GNU Library General Public License instead.)  You can apply it to
cyour programs, too.
c
c   When we speak of free software, we are referring to freedom, not
cprice.  Our General Public Licenses are designed to make sure that you
chave the freedom to distribute copies of free software (and charge for
cthis service if you wish), that you receive source code or can get it
cif you want it, that you can change the software or use pieces of it in
cnew free programs; and that you know you can do these things.
c
c   To protect your rights, we need to make restrictions that forbid
canyone to deny you these rights or to ask you to surrender the rights.
cThese restrictions translate to certain responsibilities for you if you
cdistribute copies of the software, or if you modify it.
c
c   For example, if you distribute copies of such a program, whether
cgratis or for a fee, you must give the recipients all the rights that
cyou have.  You must make sure that they, too, receive or can get the
csource code.  And you must show them these terms so they know their
crights.
c
c   We protect your rights with two steps: (1) copyright the software,
cand (2) offer you this license which gives you legal permission to copy,
cdistribute and/or modify the software.
c
c   Also, for each author's protection and ours, we want to make certain
cthat everyone understands that there is no warranty for this free
csoftware.  If the software is modified by someone else and passed on, we
cwant its recipients to know that what they have is not the original, so
cthat any problems introduced by others will not reflect on the original
cauthors' reputations.
c
c   Finally, any free program is threatened constantly by software
cpatents.  We wish to avoid the danger that redistributors of a free
cprogram will individually obtain patent licenses, in effect making the
cprogram proprietary.  To prevent this, we have made it clear that any
cpatent must be licensed for everyone's free use or not licensed at all.
c
c   The precise terms and conditions for copying, distribution and
cmodification follow.
c
c    TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
c
c  0. This License applies to any program or other work which contains a
c     notice placed by the copyright holder saying it may be distributed
c     under the terms of this General Public License.  The "Program",
c     below, refers to any such program or work, and a "work based on
c     the Program" means either the Program or any derivative work under
c     copyright law: that is to say, a work containing the Program or a
c     portion of it, either verbatim or with modifications and/or
c     translated into another language.  (Hereinafter, translation is
c     included without limitation in the term "modification".)  Each
c     licensee is addressed as "you".
c
c     Activities other than copying, distribution and modification are
c     not covered by this License; they are outside its scope.  The act
c     of running the Program is not restricted, and the output from the
c     Program is covered only if its contents constitute a work based on
c     the Program (independent of having been made by running the
c     Program).  Whether that is true depends on what the Program does.
c
c  1. You may copy and distribute verbatim copies of the Program's
c     source code as you receive it, in any medium, provided that you
c     conspicuously and appropriately publish on each copy an appropriate
c     copyright notice and disclaimer of warranty; keep intact all the
c     notices that refer to this License and to the absence of any
c     warranty; and give any other recipients of the Program a copy of
c     this License along with the Program.
c
c     You may charge a fee for the physical act of transferring a copy,
c     and you may at your option offer warranty protection in exchange
c     for a fee.
c
c  2. You may modify your copy or copies of the Program or any portion
c     of it, thus forming a work based on the Program, and copy and
c     distribute such modifications or work under the terms of Section 1
c     above, provided that you also meet all of these conditions:
c
c       a. You must cause the modified files to carry prominent notices
c          stating that you changed the files and the date of any change.
c
c       b. You must cause any work that you distribute or publish, that
c          in whole or in part contains or is derived from the Program
c          or any part thereof, to be licensed as a whole at no charge
c          to all third parties under the terms of this License.
c
c       c. If the modified program normally reads commands interactively
c          when run, you must cause it, when started running for such
c          interactive use in the most ordinary way, to print or display
c          an announcement including an appropriate copyright notice and
c          a notice that there is no warranty (or else, saying that you
c          provide a warranty) and that users may redistribute the
c          program under these conditions, and telling the user how to
c          view a copy of this License.  (Exception: if the Program
c          itself is interactive but does not normally print such an
c          announcement, your work based on the Program is not required
c          to print an announcement.)
c
c     These requirements apply to the modified work as a whole.  If
c     identifiable sections of that work are not derived from the
c     Program, and can be reasonably considered independent and separate
c     works in themselves, then this License, and its terms, do not
c     apply to those sections when you distribute them as separate
c     works.  But when you distribute the same sections as part of a
c     whole which is a work based on the Program, the distribution of
c     the whole must be on the terms of this License, whose permissions
c     for other licensees extend to the entire whole, and thus to each
c     and every part regardless of who wrote it.
c
c     Thus, it is not the intent of this section to claim rights or
c     contest your rights to work written entirely by you; rather, the
c     intent is to exercise the right to control the distribution of
c     derivative or collective works based on the Program.
c
c     In addition, mere aggregation of another work not based on the
c     Program with the Program (or with a work based on the Program) on
c     a volume of a storage or distribution medium does not bring the
c     other work under the scope of this License.
c
c  3. You may copy and distribute the Program (or a work based on it,
c     under Section 2) in object code or executable form under the terms
c     of Sections 1 and 2 above provided that you also do one of the
c     following:
c
c       a. Accompany it with the complete corresponding machine-readable
c          source code, which must be distributed under the terms of
c          Sections 1 and 2 above on a medium customarily used for
c          software interchange; or,
c
c       b. Accompany it with a written offer, valid for at least three
c          years, to give any third party, for a charge no more than your
c          cost of physically performing source distribution, a complete
c          machine-readable copy of the corresponding source code, to be
c          distributed under the terms of Sections 1 and 2 above on a
c          medium customarily used for software interchange; or,
c
c       c. Accompany it with the information you received as to the offer
c          to distribute corresponding source code.  (This alternative is
c          allowed only for noncommercial distribution and only if you
c          received the program in object code or executable form with
c          such an offer, in accord with Subsection b above.)
c
c     The source code for a work means the preferred form of the work for
c     making modifications to it.  For an executable work, complete
c     source code means all the source code for all modules it contains,
c     plus any associated interface definition files, plus the scripts
c     used to control compilation and installation of the executable.
c     However, as a special exception, the source code distributed need
c     not include anything that is normally distributed (in either
c     source or binary form) with the major components (compiler,
c     kernel, and so on) of the operating system on which the executable
c     runs, unless that component itself accompanies the executable.
c
c     If distribution of executable or object code is made by offering
c     access to copy from a designated place, then offering equivalent
c     access to copy the source code from the same place counts as
c     distribution of the source code, even though third parties are not
c     compelled to copy the source along with the object code.
c
c  4. You may not copy, modify, sublicense, or distribute the Program
c     except as expressly provided under this License.  Any attempt
c     otherwise to copy, modify, sublicense or distribute the Program is
c     void, and will automatically terminate your rights under this
c     License.  However, parties who have received copies, or rights,
c     from you under this License will not have their licenses
c     terminated so long as such parties remain in full compliance.
c
c  5. You are not required to accept this License, since you have not
c     signed it.  However, nothing else grants you permission to modify
c     or distribute the Program or its derivative works.  These actions
c     are prohibited by law if you do not accept this License.
c     Therefore, by modifying or distributing the Program (or any work
c     based on the Program), you indicate your acceptance of this
c     License to do so, and all its terms and conditions for copying,
c     distributing or modifying the Program or works based on it.
c
c  6. Each time you redistribute the Program (or any work based on the
c     Program), the recipient automatically receives a license from the
c     original licensor to copy, distribute or modify the Program
c     subject to these terms and conditions.  You may not impose any
c     further restrictions on the recipients' exercise of the rights
c     granted herein.  You are not responsible for enforcing compliance
c     by third parties to this License.
c
c  7. If, as a consequence of a court judgment or allegation of patent
c     infringement or for any other reason (not limited to patent
c     issues), conditions are imposed on you (whether by court order,
c     agreement or otherwise) that contradict the conditions of this
c     License, they do not excuse you from the conditions of this
c     License.  If you cannot distribute so as to satisfy simultaneously
c     your obligations under this License and any other pertinent
c     obligations, then as a consequence you may not distribute the
c     Program at all.  For example, if a patent license would not permit
c     royalty-free redistribution of the Program by all those who
c     receive copies directly or indirectly through you, then the only
c     way you could satisfy both it and this License would be to refrain
c     entirely from distribution of the Program.
c
c     If any portion of this section is held invalid or unenforceable
c     under any particular circumstance, the balance of the section is
c     intended to apply and the section as a whole is intended to apply
c     in other circumstances.
c
c     It is not the purpose of this section to induce you to infringe any
c     patents or other property right claims or to contest validity of
c     any such claims; this section has the sole purpose of protecting
c     the integrity of the free software distribution system, which is
c     implemented by public license practices.  Many people have made
c     generous contributions to the wide range of software distributed
c     through that system in reliance on consistent application of that
c     system; it is up to the author/donor to decide if he or she is
c     willing to distribute software through any other system and a
c     licensee cannot impose that choice.
c
c     This section is intended to make thoroughly clear what is believed
c     to be a consequence of the rest of this License.
c
c  8. If the distribution and/or use of the Program is restricted in
c     certain countries either by patents or by copyrighted interfaces,
c     the original copyright holder who places the Program under this
c     License may add an explicit geographical distribution limitation
c     excluding those countries, so that distribution is permitted only
c     in or among countries not thus excluded.  In such case, this
c     License incorporates the limitation as if written in the body of
c     this License.
c
c  9. The Free Software Foundation may publish revised and/or new
c     versions of the General Public License from time to time.  Such
c     new versions will be similar in spirit to the present version, but
c     may differ in detail to address new problems or concerns.
c
c     Each version is given a distinguishing version number.  If the
c     Program specifies a version number of this License which applies
c     to it and "any later version", you have the option of following
c     the terms and conditions either of that version or of any later
c     version published by the Free Software Foundation.  If the Program
c     does not specify a version number of this License, you may choose
c     any version ever published by the Free Software Foundation.
c
c 10. If you wish to incorporate parts of the Program into other free
c     programs whose distribution conditions are different, write to the
c     author to ask for permission.  For software which is copyrighted
c     by the Free Software Foundation, write to the Free Software
c     Foundation; we sometimes make exceptions for this.  Our decision
c     will be guided by the two goals of preserving the free status of
c     all derivatives of our free software and of promoting the sharing
c     and reuse of software generally.
c
c                                NO WARRANTY
c
c 11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO
c     WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE
c     LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
c     HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT
c     WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT
c     NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
c     FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE
c     QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
c     PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY
c     SERVICING, REPAIR OR CORRECTION.
c
c 12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN
c     WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY
c     MODIFY AND/OR REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE
c     LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL,
c     INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR
c     INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
c     DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU
c     OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY
c     OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN
c     ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
c
c                      END OF TERMS AND CONDITIONS
c
